try:
    from django.test.runner import DiscoverRunner
except ImportError:
    from django.test.simple import DjangoTestSuiteRunner as DiscoverRunner

from behave_django.environment import BehaveHooksMixin
from behave_django.testcase import (BehaviorDrivenTestCase,
                                    ExistingDatabaseTestCase)


class BehaviorDrivenTestRunner(DiscoverRunner, BehaveHooksMixin):

    """Test runner that uses the BehaviorDrivenTestCase"""
    testcase_class = BehaviorDrivenTestCase


class ExistingDatabaseTestRunner(DiscoverRunner, BehaveHooksMixin):

    """Test runner that uses the ExistingDatabaseTestCase

    This test runner nullifies Django's test database setup methods. Using this
    test runner would make your tests run with the default configured database
    in settings.py.
    """
    testcase_class = ExistingDatabaseTestCase

    def setup_databases(*args, **kwargs):
        pass

    def teardown_databases(*args, **kwargs):
        pass



try:
    from django.contrib.staticfiles.testing import StaticLiveServerTestCase
except ImportError:
    from django.test import LiveServerTestCase as StaticLiveServerTestCase


class BehaviorDrivenTestCase(StaticLiveServerTestCase):

    """Test case attached to the context during behave execution

    This test case prevents the regular tests from running.
    """

    def runTest(*args, **kwargs):
        pass


class ExistingDatabaseTestCase(BehaviorDrivenTestCase):

    """ Test case used for the --use-existing-database setup

    This test case prevents fixtures from being loaded to the database in use.
    """

    def _fixture_setup(self):
        pass

    def _fixture_teardown(self):
        pass



from optparse import OptionParser, BadOptionError, AmbiguousOptionError


class PassThroughOptionParser(OptionParser):

    """
    An unknown option pass-through implementation of OptionParser.

    When unknown arguments are encountered, bundle with largs and try again,
    until rargs is depleted.

    sys.exit(status) will still be called if a known argument is passed
    incorrectly (e.g. missing arguments or bad argument types, etc.)

    Source: http://stackoverflow.com/a/9307174
    """

    def _process_args(self, largs, rargs, values):
        while rargs:
            try:
                OptionParser._process_args(self, largs, rargs, values)
            except (BadOptionError, AmbiguousOptionError) as e:
                largs.append(e.opt_str)



from __future__ import absolute_import
from optparse import make_option
import sys

from behave.configuration import options as behave_options
from behave.__main__ import main as behave_main
from django.core.management.base import BaseCommand

from behave_django.environment import monkey_patch_behave
from behave_django.parser import PassThroughOptionParser
from behave_django.runner import (BehaviorDrivenTestRunner,
                                  ExistingDatabaseTestRunner)


def get_command_options():
    return (
        make_option(
            '--use-existing-database',
            action='store_true',
            default=False,
            help="Don't create a test database. USE AT YOUR OWN RISK!",
        ),
    )


def get_behave_options():
    """Creates options for the behave management command based on behave"""

    new_options = []

    conflicts = [
        '--no-color',
        '--version'
    ]

    for fixed, keywords in behave_options:
        # TODO: accept short options too
        long_option = None
        for option in fixed:
            if option.startswith("--"):
                long_option = option
                break

        # Do not add conflicting options
        if long_option in conflicts:
            continue

        if long_option:
            # type isn't a valid keyword for make_option
            if hasattr(keywords.get('type'), '__call__'):
                del keywords['type']
            # config_help isn't a valid keyword for make_option
            if 'config_help' in keywords:
                del keywords['config_help']

            new_options.append(
                make_option(long_option, **keywords)
            )
    return tuple(new_options)


class Command(BaseCommand):
    help = 'Runs behave tests'
    option_list = BaseCommand.option_list + get_behave_options() + \
        get_command_options()

    def handle(self, *args, **options):
        behave_args = self.get_behave_args()

        # Configure django environment
        if options['dry_run'] or options['use_existing_database']:
            django_test_runner = ExistingDatabaseTestRunner()
        else:
            django_test_runner = BehaviorDrivenTestRunner()
        django_test_runner.setup_test_environment()
        old_config = django_test_runner.setup_databases()

        # Run Behave tests
        monkey_patch_behave(django_test_runner)
        exit_status = behave_main(args=behave_args)

        # Teardown django environment
        django_test_runner.teardown_databases(old_config)
        django_test_runner.teardown_test_environment()

        if exit_status != 0:
            sys.exit(exit_status)

    def get_behave_args(self, argv=sys.argv):
        """Remove command line arguments not accepted by behave."""
        # Override option_list to remove the behave arguments
        orig_option_list = self.option_list
        self.option_list = BaseCommand.option_list + get_command_options()
        # Get an OptionParser from django with the new option_list
        django_parser = self.create_parser('manage.py', 'behave')

        # Put back the original option_list to minimize side effets
        self.option_list = orig_option_list

        # Load the option_list generated by django into our custom parser
        parser = PassThroughOptionParser()
        for option in django_parser.option_list:
            if '--help' not in option._long_opts:
                parser.add_option(option)

        # Our custom parser returns the unrecognized args instead of exploding
        options, args = parser.parse_args(argv[2:])

        # The unrecognized args are for behave :)
        return args



import warnings

from behave.runner import ModelRunner
from django.core.management import call_command

try:
    from django.shortcuts import resolve_url
except ImportError:
    # support Django 1.4, which has no resolve_url() yet
    from django.shortcuts import redirect
    resolve_url = lambda to, *args, **kwargs: \
        redirect(to, *args, **kwargs)['Location']


class BehaveHooksMixin(object):

    """Provides methods that run during test execution

    These methods are attached to behave via monkey patching.
    """
    testcase_class = None

    def before_scenario(self, context):
        """Method that runs before behave's before_scenario function

        Sets up the test case, base_url, and the get_url() utility function.
        """
        context.test = self.testcase_class()
        context.test.setUpClass()
        context.test()

        context.base_url = context.test.live_server_url

        def get_url(to=None, *args, **kwargs):
            return context.base_url + (
                resolve_url(to, *args, **kwargs) if to else '')

        context.get_url = get_url

    def load_fixtures(self, context):
        """Method that runs immediately after behave's before_scenario function

        If fixtures are found in context, loads the fixtures using the loaddata
        management command.
        """
        if getattr(context, 'fixtures', None):
            call_command('loaddata', *context.fixtures, verbosity=0)

    def after_scenario(self, context):
        """Method that runs immediately after behave's after_scenario function
        """
        context.test.tearDownClass()
        del context.test


def monkey_patch_behave(django_test_runner):
    """Integrate behave_django in behave via before/after scenario hooks."""
    behave_run_hook = ModelRunner.run_hook

    def run_hook(self, name, context, *args):
        if name == 'before_scenario':
            django_test_runner.before_scenario(context)
        behave_run_hook(self, name, context, *args)
        if name == 'before_scenario':
            django_test_runner.load_fixtures(context)
        if name == 'after_scenario':
            django_test_runner.after_scenario(context)

    ModelRunner.run_hook = run_hook


def before_scenario(*args, **kwargs):
    warnings.warn("DEPRECATED: This function is no longer needed.",
                  stacklevel=2)


def after_scenario(*args, **kwargs):
    warnings.warn("DEPRECATED: This function is no longer needed.",
                  stacklevel=2)
